/****************************************************************************** 
StoneFlower & Anatoly Berezkin
March 2018

Development environment specifics:
Written in Arduino 1.6.0
*****************************************************************************/
#include <TMC2130Stepper.h>
#include <TMC2130Stepper_REGDEFS.h>
#include <TMC2130Stepper_UTILITY.h>
#include <SPI.h>

//Declare pin functions on board
#define EN_PIN  2
#define DIR_PIN  4
#define STP_PIN 3
#define CS_PIN  10
#define MOSI_PIN  11
#define MISO_PIN 12
#define SCK_PIN  13
#define ph_ms1 A3
#define ph_ms2 A4
#define ph_ms3 A5
#define push_btn 6 //if digital pin D6 is low, we rotate the motor (switch J6)

//TMC2130Stepper driver = TMC2130Stepper(EN_PIN, DIR_PIN, STP_PIN, CS_PIN, MOSI_PIN, MISO_PIN, SCK_PIN);
TMC2130Stepper driver = TMC2130Stepper(EN_PIN, DIR_PIN, STP_PIN, CS_PIN);

//Declare variables for functions
char user_input;
int x;
int y;
int state;
int microstepping;
int feed_in;
int duration;
float spd;

void setup() {
  Serial.begin(9600);
  while(!Serial);
  pinMode(STP_PIN, OUTPUT);
  pinMode(EN_PIN, OUTPUT);
  pinMode(DIR_PIN, OUTPUT);
  pinMode(CS_PIN, OUTPUT);
//control pins of A4988 driver
  pinMode(ph_ms1, OUTPUT);
  pinMode(ph_ms2, OUTPUT);
  pinMode(ph_ms3, OUTPUT);
  pinMode(push_btn, INPUT);
  
  SPI.begin();
  pinMode(MISO_PIN, INPUT_PULLUP);
 
  driver.begin();            // Initiate pins and registers
  driver.rms_current(900);  // Set stepper current to 1000mA. The command is the same as command TMC2130.setCurrent(900, 0.11, 0.5);
  driver.push();
  driver.microsteps(4);
  //driver.stealthChop(1);     // enable quiet stepping

  driver.irun(16);
  driver.ihold(10);
  driver.blank_time(24); // [16, 24, 36, 54]
  driver.off_time(2);    // [1..15]
  driver.hysteresis_start(1);
  driver.hysteresis_end(8);

  digitalWrite(STP_PIN, LOW);
  digitalWrite(EN_PIN, LOW);  
  digitalWrite(ph_ms1, HIGH);
  digitalWrite(ph_ms2, HIGH);
  digitalWrite(ph_ms3, HIGH);
}

//Main loop
void loop() {

    state = digitalRead(push_btn); //Read user input and trigger appropriate function
    microstepping = int(analogRead(A6)/205);
    switch (microstepping) {
      case 0:// 1
          digitalWrite(ph_ms1, LOW);
          digitalWrite(ph_ms2, LOW);
          digitalWrite(ph_ms3, LOW);
          break;
      case 1:// 1/2
          digitalWrite(ph_ms1, HIGH);
          digitalWrite(ph_ms2, LOW);
          digitalWrite(ph_ms3, LOW);
          break;
      case 2:// 1/4
          digitalWrite(ph_ms1, LOW);
          digitalWrite(ph_ms2, HIGH);
          digitalWrite(ph_ms3, LOW);
          break;
      case 3:// 1/8
          digitalWrite(ph_ms1, HIGH);
          digitalWrite(ph_ms2, HIGH);
          digitalWrite(ph_ms3, LOW);
          break;
      case 4:// 1/16
          digitalWrite(ph_ms1, HIGH);
          digitalWrite(ph_ms2, HIGH);
          digitalWrite(ph_ms3, HIGH);
          break;
      default:// 1/16
          digitalWrite(ph_ms1, HIGH);
          digitalWrite(ph_ms2, HIGH);
          digitalWrite(ph_ms3, HIGH);
    }
    feed_in = 1024-analogRead(A7);
    spd = 4.9+feed_in/1024.0*195.0; //speed from voltage ml/h
    duration = 303.746*60.0/spd;   //ram S

    if (state == LOW & spd > 5.0)
      {
           StepDefault();
      }
}

//Reset Easy Driver pins to default states

//Default microstep mode function
void StepDefault()
{
  for(x= 1; x<100; x++)  //Loop the stepping
  {
    PORTD = PORTD | B00001000; //set STP_PIN=3 to HIGH, to trigger one step forward. Other pins are unaffected.
    delayMicroseconds(duration);
    PORTD = PORTD & B11110111; //set STP_PIN=3 to LOW, so it can be triggered again. Other pins are unaffected.
    delayMicroseconds(duration);
  }
}
